@@ -32,6 +32,9 @@
 #include "pdns/packetcache.hh"
 #include "dnspacket.hh"
 #include "dns.hh"
+#include "statbag.hh"
+
+extern StatBag S;
 
 // this has to be somewhere central, and not in a file that requires Lua
 // this is so the geoipbackend can set this pointer if loaded for lua-record.cc
@@ -85,6 +88,15 @@ void BackendMakerClass::report(BackendFactory *bf)
   d_repository[bf->getName()]=bf;
 }
 
+void BackendMakerClass::clear()
+{
+  d_instances.clear();
+  for (auto& repo : d_repository) {
+    delete repo.second;
+    repo.second = nullptr;
+  }
+  d_repository.clear();
+}
 
 vector<string> BackendMakerClass::getModules()
 {
@@ -139,7 +151,7 @@ void BackendMakerClass::launch(const string &instr)
   vector<string> parts;
   stringtok(parts,instr,", ");
 
-  for (const auto part : parts)
+  for (const auto& part : parts)
     if (count(parts.begin(), parts.end(), part) > 1)
       throw ArgException("Refusing to launch multiple backends with the same name '" + part + "', verify all 'launch' statements in your configuration");
 
@@ -164,41 +176,50 @@ void BackendMakerClass::launch(const string &instr)
   }
 }
 
-int BackendMakerClass::numLauncheable()
+size_t BackendMakerClass::numLauncheable() const
 {
   return d_instances.size();
 }
 
-vector<DNSBackend *>BackendMakerClass::all(bool metadataOnly)
+vector<DNSBackend *> BackendMakerClass::all(bool metadataOnly)
 {
-  vector<DNSBackend *>ret;
+  vector<DNSBackend *> ret;
   if(d_instances.empty())
     throw PDNSException("No database backends configured for launch, unable to function");
 
+  ret.reserve(d_instances.size());
+
   try {
-    for(vector<pair<string,string> >::const_iterator i=d_instances.begin();i!=d_instances.end();++i) {
-      DNSBackend *made;
-      if(metadataOnly)
-        made = d_repository[i->first]->makeMetadataOnly(i->second);
-      else
-        made = d_repository[i->first]->make(i->second);
-      if(!made)
-        throw PDNSException("Unable to launch backend '"+i->first+"'");
+    for (const auto& instance : d_instances) {
+      DNSBackend *made = nullptr;
+
+      if (metadataOnly) {
+        made = d_repository[instance.first]->makeMetadataOnly(instance.second);
+      }
+      else {
+        made = d_repository[instance.first]->make(instance.second);
+      }
+
+      if (!made) {
+        throw PDNSException("Unable to launch backend '" + instance.first + "'");
+      }
 
       ret.push_back(made);
     }
   }
-  catch(PDNSException &ae) {
+  catch(const PDNSException &ae) {
     g_log<<Logger::Error<<"Caught an exception instantiating a backend: "<<ae.reason<<endl;
     g_log<<Logger::Error<<"Cleaning up"<<endl;
-    for(vector<DNSBackend *>::const_iterator i=ret.begin();i!=ret.end();++i)
-      delete *i;
+    for (auto i : ret) {
+      delete i;
+    }
     throw;
   } catch(...) {
     // and cleanup
     g_log<<Logger::Error<<"Caught an exception instantiating a backend, cleaning up"<<endl;
-    for(vector<DNSBackend *>::const_iterator i=ret.begin();i!=ret.end();++i)
-      delete *i;
+    for (auto i : ret) {
+      delete i;
+    }
     throw;
   }
 
@@ -223,6 +244,7 @@ vector<DNSBackend *>BackendMakerClass::all(bool metadataOnly)
 bool DNSBackend::getSOA(const DNSName &domain, SOAData &sd)
 {
   this->lookup(QType(QType::SOA),domain,-1);
+  S.inc("backend-queries");
 
   DNSResourceRecord rr;
   rr.auth = true;
@@ -239,20 +261,10 @@ bool DNSBackend::getSOA(const DNSName &domain, SOAData &sd)
 
   if(!hits)
     return false;
+
   sd.qname = domain;
-  if(!sd.nameserver.countLabels())
-    sd.nameserver= DNSName(arg()["default-soa-name"]);
-
-  if(!sd.hostmaster.countLabels()) {
-    if (!arg().isEmpty("default-soa-mail")) {
-      sd.hostmaster= DNSName(arg()["default-soa-mail"]);
-      // attodot(sd.hostmaster); FIXME400
-    }
-    else
-      sd.hostmaster=DNSName("hostmaster")+domain;
-  }
-
   sd.db=this;
+
   return true;
 }
 
@@ -267,37 +279,12 @@ bool DNSBackend::get(DNSZoneRecord& dzr)
   dzr.scopeMask = rr.scopeMask;
   if(rr.qtype.getCode() == QType::TXT && !rr.content.empty() && rr.content[0]!='"')
     rr.content = "\""+ rr.content + "\"";
-  if(rr.qtype.getCode() == QType::SOA) {
-    try {
-      dzr.dr = DNSRecord(rr);
-    } catch(...) {
-      vector<string> parts;
-      stringtok(parts, rr.content, " \t");
-      if(parts.size() < 1)
-        rr.content = arg()["default-soa-name"];
-      if(parts.size() < 2)
-        rr.content += " " +arg()["default-soa-mail"];
-      if(parts.size() < 3)
-        rr.content += " 0";
-      if(parts.size() < 4)
-        rr.content += " " + ::arg()["soa-refresh-default"];
-      if(parts.size() < 5)
-        rr.content += " " + ::arg()["soa-retry-default"];
-      if(parts.size() < 6)
-        rr.content += " " + ::arg()["soa-expire-default"];
-      if(parts.size() < 7)
-        rr.content += " " + ::arg()["soa-minimum-ttl"];
-      dzr.dr = DNSRecord(rr);
-    }
+  try {
+    dzr.dr = DNSRecord(rr);
   }
-  else {
-    try {
-      dzr.dr = DNSRecord(rr);
-    }
-    catch(...) {
-      while(this->get(rr));
-      throw;
-    }
+  catch(...) {
+    while(this->get(rr));
+    throw;
   }
   return true;
 }
@@ -338,41 +325,20 @@ std::shared_ptr<DNSRecordContent> makeSOAContent(const SOAData& sd)
     return std::make_shared<SOARecordContent>(sd.nameserver, sd.hostmaster, st);
 }
 
-
 void fillSOAData(const string &content, SOAData &data)
 {
-  // content consists of fields separated by spaces:
-  //  nameservername hostmaster serial-number [refresh [retry [expire [ minimum] ] ] ]
-
-  // fill out data with some plausible defaults:
-  // 10800 3600 604800 3600
   vector<string>parts;
   parts.reserve(7);
-  stringtok(parts,content);
-  int pleft=parts.size();
-
-  //  cout<<"'"<<content<<"'"<<endl;
-
-  if(pleft)
-    data.nameserver=DNSName(parts[0]);
-
-  if(pleft>1) 
-    data.hostmaster=DNSName(attodot(parts[1])); // ahu@ds9a.nl -> ahu.ds9a.nl, piet.puk@ds9a.nl -> piet\.puk.ds9a.nl
+  stringtok(parts, content);
 
   try {
-    data.serial = pleft > 2 ? pdns_stou(parts[2]) : 0;
-
-    data.refresh = pleft > 3 ? pdns_stou(parts[3])
-      : ::arg().asNum("soa-refresh-default");
-
-    data.retry = pleft > 4 ? pdns_stou(parts[4].c_str())
-      : ::arg().asNum("soa-retry-default");
-
-    data.expire = pleft > 5 ? pdns_stou(parts[5].c_str())
-      : ::arg().asNum("soa-expire-default");
-
-    data.minimum = pleft > 6 ? pdns_stou(parts[6].c_str())
-      : ::arg().asNum("soa-minimum-ttl");
+    data.nameserver = DNSName(parts.at(0));
+    data.hostmaster = DNSName(attodot(parts.at(1))); // ahu@ds9a.nl -> ahu.ds9a.nl, piet.puk@ds9a.nl -> piet\.puk.ds9a.nl
+    data.serial = pdns_stou(parts.at(2).c_str());
+    data.refresh = pdns_stou(parts.at(3).c_str());
+    data.retry = pdns_stou(parts.at(4).c_str());
+    data.expire = pdns_stou(parts.at(5).c_str());
+    data.minimum = pdns_stou(parts.at(6).c_str());
   }
   catch(const std::out_of_range& oor) {
     throw PDNSException("Out of range exception parsing "+content);
