@@ -53,9 +53,9 @@ DNSPacketWriter::DNSPacketWriter(vector<uint8_t>& content, const DNSName& qname,
   dnsheader.opcode=opcode;
 
   const uint8_t* ptr=(const uint8_t*)&dnsheader;
-  uint32_t len=d_content.size();
-  d_content.resize(len + sizeof(dnsheader));
-  uint8_t* dptr=(&*d_content.begin()) + len;
+  d_content.reserve(sizeof(dnsheader) + qname.wirelength() + sizeof(qtype) + sizeof(qclass));
+  d_content.resize(sizeof(dnsheader));
+  uint8_t* dptr=(&*d_content.begin());
 
   memcpy(dptr, ptr, sizeof(dnsheader));
   d_namepositions.reserve(16);
@@ -388,6 +388,63 @@ void DNSPacketWriter::xfrHexBlob(const string& blob, bool keepReading)
   xfrBlob(blob);
 }
 
+void DNSPacketWriter::xfrSvcParamKeyVals(const std::set<SvcParam> &kvs)
+{
+  for (auto const &param : kvs) {
+    // Key first!
+    xfr16BitInt(param.getKey());
+
+    switch (param.getKey())
+    {
+    case SvcParam::mandatory:
+      xfr16BitInt(2 * param.getMandatory().size());
+      for (auto const &m: param.getMandatory()) {
+        xfr16BitInt(m);
+      }
+      break;
+    case SvcParam::alpn:
+    {
+      uint16_t totalSize = param.getALPN().size(); // All 1 octet size headers for each value
+      for (auto const &a : param.getALPN()) {
+        totalSize += a.length();
+      }
+      xfr16BitInt(totalSize);
+      for (auto const &a : param.getALPN()) {
+        xfrUnquotedText(a, true); // will add the 1-byte length field
+      }
+      break;
+    }
+    case SvcParam::no_default_alpn:
+      xfr16BitInt(0); // no size :)
+      break;
+    case SvcParam::port:
+      xfr16BitInt(2); // size
+      xfr16BitInt(param.getPort());
+      break;
+    case SvcParam::ipv4hint:
+      xfr16BitInt(param.getIPHints().size() * 4); // size
+      for (auto a: param.getIPHints()) {
+        xfrCAWithoutPort(param.getKey(), a);
+      }
+      break;
+    case SvcParam::ipv6hint:
+      xfr16BitInt(param.getIPHints().size() * 16); // size
+      for (auto a: param.getIPHints()) {
+        xfrCAWithoutPort(param.getKey(), a);
+      }
+      break;
+    case SvcParam::echconfig:
+      xfr16BitInt(param.getEchConfig().size()); // size
+      xfrBlobNoSpaces(param.getEchConfig());
+      break;
+    default:
+      xfr16BitInt(param.getValue().size());
+      xfrBlob(param.getValue());
+      break;
+    }
+  }
+}
+
 // call __before commit__
 void DNSPacketWriter::getRecordPayload(string& records)
 {
