@@ -18,24 +18,23 @@ struct Question
   DTime d_dt;
   DNSName qdomain;
   QType qtype;
-  DNSPacket* replyPacket()
+  std::unique_ptr<DNSPacket> replyPacket()
   {
-    return new DNSPacket(false);
+    return make_unique<DNSPacket>(false);
   }
 };
 
 struct Backend
 {
-  DNSPacket* question(Question*)
+  std::unique_ptr<DNSPacket> question(Question&)
   {
-    return new DNSPacket(true);
+    return make_unique<DNSPacket>(true);
   }
 };
 
 static std::atomic<int> g_receivedAnswers;
-static void report(DNSPacket* A)
+static void report(std::unique_ptr<DNSPacket>& A)
 {
-  delete A;
   g_receivedAnswers++;
 }
 
@@ -50,8 +49,8 @@ BOOST_AUTO_TEST_CASE(test_distributor_basic) {
 
   int n;
   for(n=0; n < 100; ++n)  {
-    auto q = new Question();
-    q->d_dt.set(); 
+    Question q;
+    q.d_dt.set(); 
     d->question(q, report);
   }
   sleep(1);
@@ -60,17 +59,16 @@ BOOST_AUTO_TEST_CASE(test_distributor_basic) {
 
 struct BackendSlow
 {
-  DNSPacket* question(Question*)
+  std::unique_ptr<DNSPacket> question(Question&)
   {
     sleep(1);
-    return new DNSPacket(true);
+    return make_unique<DNSPacket>(true);
   }
 };
 
 static std::atomic<int> g_receivedAnswers1;
-static void report1(DNSPacket* A)
+static void report1(std::unique_ptr<DNSPacket>& A)
 {
-  delete A;
   g_receivedAnswers1++;
 }
 
@@ -87,8 +85,8 @@ BOOST_AUTO_TEST_CASE(test_distributor_queue) {
     int n;
     // bound should be higher than max-queue-length
     for(n=0; n < 2000; ++n)  {
-      auto q = new Question();
-      q->d_dt.set(); 
+      Question q;
+      q.d_dt.set(); 
       d->question(q, report1);
     }
     }, DistributorFatal, [](DistributorFatal) { return true; });
@@ -103,14 +101,14 @@ struct BackendDies
   ~BackendDies()
   {
   }
-  DNSPacket* question(Question* q)
+  std::unique_ptr<DNSPacket> question(Question& q)
   {
     //  cout<<"Q: "<<q->qdomain<<endl;
     if(!d_ourcount && ++d_count == 10) {
       // cerr<<"Going.. down!"<<endl;
       throw runtime_error("kill");
     }
-    return new DNSPacket(true);
+    return make_unique<DNSPacket>(true);
   }
   static std::atomic<int> s_count;
   int d_count{0};
@@ -121,9 +119,8 @@ std::atomic<int> BackendDies::s_count;
 
 std::atomic<int> g_receivedAnswers2;
 
-static void report2(DNSPacket* A)
+static void report2(std::unique_ptr<DNSPacket>& A)
 {
-  delete A;
   g_receivedAnswers2++;
 }
 
@@ -139,10 +136,10 @@ BOOST_AUTO_TEST_CASE(test_distributor_dies) {
 
   try {
     for(int n=0; n < 100; ++n)  {
-      auto q = new Question();
-      q->d_dt.set(); 
-      q->qdomain=DNSName(std::to_string(n));
-      q->qtype = QType(QType::A);
+      Question q;
+      q.d_dt.set(); 
+      q.qdomain=DNSName(std::to_string(n));
+      q.qtype = QType(QType::A);
       d->question(q, report2);
     }
 
