@@ -19,7 +19,7 @@ ArgvMap &arg()
 }
 
 
-bool init_unit_test() {
+static bool init_unit_test() {
   reportAllTypes();
   return true;
 }
@@ -27,5 +27,6 @@ bool init_unit_test() {
 // entry point:
 int main(int argc, char* argv[])
 {
+  S.d_allowRedeclare = true;
   return boost::unit_test::unit_test_main( &init_unit_test, argc, argv );
 }
